package org.zeroturnaround.javarebel.integration.util;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import org.zeroturnaround.javarebel.ClassResource;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.ResourceSource;

/**
 * Helper methods for finding resources.
 * 
 * @author Rein Raudjärv
 */
public class ResourceUtil {

  private static final Method FIND_RESOURCE_METHOD  = getClassLoaderMethod("findResource", new Class[] {String.class});
  private static final Method FIND_RESOURCES_METHOD = getClassLoaderMethod("findResources", new Class[] {String.class});
  
  private static Method getClassLoaderMethod(String methodName, Class[] paramTypes) {
    Method result = null;
    try {
      result = ClassLoader.class.getDeclaredMethod(methodName, paramTypes);
      result.setAccessible(true);
    } 
    catch (NoSuchMethodException e) {
      LoggerFactory.getInstance().errorEcho(e);
    }
    return result;
  }
  
  public static URL findResource(ClassLoader classloader, String name) {
    URL url;
    if (classloader instanceof URLClassLoader)
      url = ((URLClassLoader) classloader).findResource(name);
    else {
      try {
        url = (URL) FIND_RESOURCE_METHOD.invoke(classloader, new Object[] {name});
      } catch (IllegalArgumentException e) {
        throw new RuntimeException(e);
      } catch (IllegalAccessException e) {
        throw new RuntimeException(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(e.getTargetException());
      }
    }
    return url;
  }
  
  public static URL[] findResources(ClassLoader classloader, String name) {
    Enumeration en;
    if (classloader instanceof URLClassLoader)
      try {
        en = ((URLClassLoader) classloader).findResources(name);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    else {
      try {
        en = (Enumeration) FIND_RESOURCES_METHOD.invoke(classloader, new Object[] {name});
      } catch (IllegalArgumentException e) {
        throw new RuntimeException(e);
      } catch (IllegalAccessException e) {
        throw new RuntimeException(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(e.getTargetException());
      }
    }
    
    if (en == null || !en.hasMoreElements())
      return null;
    
    return toURLs(en);
  }
  
  public static URL[] findResources(URLClassLoader classloader, String name) {
    Enumeration en;
    try {
      en = classloader.findResources(name);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    
    if (en == null || !en.hasMoreElements())
      return null;
    
    return toURLs(en);
  }
  
  public static URL[] toURLs(Enumeration en) {
    List urls = new ArrayList();
    while (en.hasMoreElements())
      urls.add((URL) en.nextElement());
    
    return (URL[]) urls.toArray(new URL[urls.size()]);
  }
  
  public static Enumeration toEnumeration(URL[] urls) {
	  return new Vector(Arrays.asList(urls)).elements();
  }
  
  public static ClassResource getClassResource(ResourceSource source, String className) {
    // Most containers don't like when JavaRebel instruments their code
    if (className.startsWith("weblogic.")
        || className.startsWith("com.evermind.")
        || className.startsWith("oracle.")
        || className.startsWith("com.bea.")
        || className.startsWith("org.eclipse.osgi.")
        || className.startsWith("com.ibm.websphere")
        || className.startsWith("org.apache.jsp."))
      return null;
    
    return source.getLocalResource(className.replace('.', '/') + ".class");
  }

  public static ClassResource[] asArray(ClassResource resource) {
    if (resource == null)
      return null;
    return new ClassResource[] { resource };
  }
  
  public static ClassResource getResource(ResourceSource aSource, Collection otherSources, String name) {
    ClassResource result = aSource.getLocalResource(name);
    if (result != null)
      return result;
    
    for (Iterator it = otherSources.iterator(); it.hasNext();) {
      ResourceSource rs = (ResourceSource) it.next();
      result = rs.getLocalResource(name);
      if (result != null)
        return result;
    }
    
    return null;
  }
    
  public static ClassResource[] getResources(ResourceSource aSource, Collection otherSources, String name) {
    List result = new ArrayList();
    
    {
      ClassResource[] resources = aSource.getLocalResources(name);
      if (resources != null)
        result.addAll(Arrays.asList(resources));
    }
    
    for (Iterator it = otherSources.iterator(); it.hasNext();) {
      ClassResource[] resources = ((ResourceSource) it.next()).getLocalResources(name);
      if (resources != null)
        result.addAll(Arrays.asList(resources));
    }
    
    if (result.isEmpty())
      return null;
    
    return (ClassResource[]) result.toArray(new ClassResource[result.size()]);
  }

}
